@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: RFC1867Parser.h 638 2005-09-07 21:51:26Z svn $
+ * $Id: RFC1867Parser.h 682 2005-09-15 06:41:06Z svn $
  *****************************************************************************/
 
 #ifndef RFC1867PARSER_H
@@ -84,13 +84,15 @@ public:
         string name;
         string tmp_path;
         string mime;
+        string digest;
         apr_size_t size;
 
         FileContent(const char *name, const char *tmp_path,
-                    const char *mime, apr_size_t size)
+                    const char *mime, const char *digest, apr_size_t size)
             : name(name),
               tmp_path(tmp_path),
               mime(mime),
+              digest(digest),
               size(size)
         {
 
@@ -100,6 +102,7 @@ public:
             : name(),
               tmp_path(),
               mime(),
+              digest(),
               size(0)
         {
 
@@ -196,7 +199,7 @@ private:
 
     bool get_content(handle& h, string& name, rfc1867_content& content);
     const char *get_text_content(handle& h);
-    const char *get_file_content(handle& h, apr_size_t *file_size);
+    const char *get_file_content(handle& h, const char **file_digest, apr_size_t *file_size);
     apr_size_t store_buffer(handle& h);
 
     static apr_size_t read(handle& h, char *buffer);
