@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploaderConfig.h 643 2005-09-07 23:36:20Z svn $
+ * $Id: UploaderConfig.h 674 2005-09-13 18:31:36Z svn $
  *****************************************************************************/
 
 #ifndef UPLOADERCONFIG_H
@@ -140,7 +140,7 @@ private:
     void init_lock();
 #endif
     void load_list();
-    void update_list();
+    void update_list(apr_pool_t *pool);
     void load_template(apr_pool_t *pool, apr_pool_t **tmpl_pool,
                        const char *template_path, page_template *tmpl);
 
