@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: Auxiliary.c 657 2005-09-08 14:50:37Z svn $
+ * $Id: Auxiliary.c 674 2005-09-13 18:31:36Z svn $
  *****************************************************************************/
 
 #include "Auxiliary.h"
@@ -37,17 +37,6 @@
 #include <ctype.h>
 
 
-apr_time_t mtime(apr_pool_t *pool, const char *path)
-{
-    apr_finfo_t info;
-
-    if (UNLIKELY(apr_stat(&info, path, APR_FINFO_MTIME, pool) != APR_SUCCESS)) {
-        return 0;
-    }
-
-    return info.mtime;
-}
-
 const char *rfc1738_encode(apr_pool_t *pool, const char* str)
 {
     static char hex[] = "0123456789ABCDEF";
@@ -89,7 +78,7 @@ const char *size_str(apr_pool_t *pool, apr_uint64_t size)
     }
 #else
     if (size > 1024*1024*1024) {
-        return apr_psprintf(pool, "%.1f %s", (double)(size/1024*1024)/1024, GBYTE_UNIT);
+        return apr_psprintf(pool, "%.1f %s", (double)(size/(1024*1024))/1024, GBYTE_UNIT);
     } else if (size > 1024*1024) {
         return apr_psprintf(pool, "%.1f %s", (double)(size/1024)/1024, MBYTE_UNIT);
     } else if (size > 1024) {
