@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: PlainConverter.cpp 517 2005-08-26 07:19:12Z svn $
+ * $Id: PlainConverter.cpp 678 2005-09-14 20:40:24Z svn $
  *****************************************************************************/
 
 #include "PlainConverter.h"
@@ -32,6 +32,7 @@
 #include "UploadItem.h"
 #include "CharCodeConverter.h"
 #include "Auxiliary.h"
+#include "Misc.h"
 
 #include "apr_strings.h"
 
@@ -134,13 +135,17 @@ void PlainConverter::convert_file(UploadItemReader &reader,
             throw "ԡΥե򳫤ޤǤ";
         }
 
-        while (apr_file_read(upload_file, buffer, &size) == APR_SUCCESS) {
-            if (apr_file_write(plain_file, buffer, &size) != APR_SUCCESS) {
+        while (apr_file_read_full(upload_file, buffer, size, &size) == APR_SUCCESS) {
+            if (apr_file_write_full(plain_file, buffer, size, &size) != APR_SUCCESS) {
                 throw "ԡΥե˽񤭹ޤǤ";
             }
             size = sizeof(buffer);
         }
 
+        if (apr_file_eof(upload_file) != APR_EOF) {
+            throw "åץɥեɤ߽Ф˼Ԥޤ";
+        }
+
         apr_file_close(plain_file);
         apr_file_close(upload_file);
     } catch(const char *message) {
