@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ThumbnailWriter.cpp 594 2005-09-05 16:38:21Z svn $
+ * $Id: ThumbnailWriter.cpp 693 2005-09-17 09:23:20Z svn $
  *****************************************************************************/
 
 #include "ThumbnailWriter.h"
@@ -52,8 +52,6 @@
 #undef PACKAGE_TARNAME
 #undef PACKAGE_VERSION
 
-#include <stdexcept>
-
 #ifdef DEBUG
 #include <iostream>
 #endif
@@ -93,7 +91,7 @@ void ThumbnailWriter::write(const char *file_path) const
         Magick::Image image(data);
         Magick::Geometry size = image.size();
 
-        if ((size.width() == 0) || (size.height() == 0)) { // ȽǴϤɤΤ?
+        if ((size.width() == 0) || (size.height() == 0)) { // ȽǴϤɤ
             throw "ͥ벽бƤʤΥեǤ";
         }
 
@@ -103,6 +101,9 @@ void ThumbnailWriter::write(const char *file_path) const
         image.write(create_thumb_path(pool, thumb_dir_, file_path));
 
         apr_pool_destroy(pool);
+    } catch(Magick::Exception &e) {
+        apr_pool_destroy(pool);
+        throw e.what();
     } catch(exception &e) {
         apr_pool_destroy(pool);
         throw e.what();
@@ -141,8 +142,7 @@ void *ThumbnailWriter::read(apr_pool_t *pool, const char *file_path, apr_size_t
 
     APR_PALLOC(buffer, void *, pool, header->file_size);
 
-    read_size = header->file_size;
-    if ((apr_file_read(file, buffer, &read_size) != APR_SUCCESS) ||
+    if ((apr_file_read_full(file, buffer, header->file_size, &read_size) != APR_SUCCESS) ||
         (read_size != header->file_size)) {
         throw "åץɥեɤ߹ޤǤ";
     }
@@ -171,7 +171,6 @@ void ThumbnailWriter::calc_size(Magick::Geometry &size) const
         }
     }
 }
-#endif
 
 
 /******************************************************************************
@@ -216,6 +215,8 @@ int main(int argc, const char * const *argv)
 }
 #endif
 
+#endif
+
 // Local Variables:
 // mode: c++
 // buffer-file-coding-system: euc-japan-dos
