@@ -24,12 +24,11 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemList.cpp 665 2005-09-09 00:56:59Z svn $
+ * $Id: UploadItemList.cpp 685 2005-09-15 07:18:18Z svn $
  *****************************************************************************/
 
 #include "UploadItemList.h"
 #include "TemplateVariableCreator.h"
-#include "Auxiliary.h"
 #include "Misc.h"
 
 #include "apr_strings.h"
@@ -58,6 +57,8 @@ UploadItemList::UploadItemList(apr_pool_t *pool,
       data_pool_(NULL),
       file_dir_(file_dir),
       thumb_dir_(thumb_dir),
+      path_cache_path_(create_path(pool, file_dir, PATH_CACHE_FILENAME)),
+      header_cache_path_(create_path(pool, file_dir, HEADER_CACHE_FILENAME)),
       total_number_limit_(total_number_limit),
       total_size_limit_(total_size_limit),
       reader_(pool, file_dir),
@@ -124,12 +125,24 @@ apr_time_t UploadItemList::get_mtime() const
 
 void UploadItemList::update_mtime()
 {
-    mtime_ = mtime(pool_, file_dir_);
+    apr_finfo_t info;
+
+    if (UNLIKELY(apr_stat(&info, file_dir_, APR_FINFO_MTIME, pool_) != APR_SUCCESS)) {
+        return;
+    }
+
+    mtime_ =  info.mtime;
 }
 
-bool UploadItemList::is_need_update() const
+bool UploadItemList::is_need_update(apr_pool_t *pool) const
 {
-    return mtime_ != mtime(pool_, file_dir_);
+    apr_finfo_t info;
+
+    if (UNLIKELY(apr_stat(&info, file_dir_, APR_FINFO_MTIME, pool) != APR_SUCCESS)) {
+        return false;
+    }
+
+    return mtime_ != info.mtime;
 }
 
 void UploadItemList::remove(const char *file_path, const char *password)
@@ -479,11 +492,12 @@ void UploadItemList::remove(item_info *info)
 }
 
 
-const char *UploadItemList::create_path(apr_pool_t *tmp_pool, const char *name)
+const char *UploadItemList::create_path(apr_pool_t *tmp_pool, const char *dir_path,
+                                        const char *file_name)
 {
     char *file_path;
 
-    if (apr_filepath_merge(&file_path, file_dir_, name,
+    if (apr_filepath_merge(&file_path, dir_path, file_name,
                            APR_FILEPATH_NOTABOVEROOT,tmp_pool) != APR_SUCCESS) {
         throw "åե̾ǤޤǤ";
     }
@@ -492,14 +506,16 @@ const char *UploadItemList::create_path(apr_pool_t *tmp_pool, const char *name)
 }
 
 const char *UploadItemList::create_tmp_path(apr_pool_t *tmp_pool, const char *name,
-                                            apr_file_t **tmp_file)
+                                            bool is_buffered, apr_file_t **tmp_file)
 {
     char *tmp_path;
 
-    tmp_path = const_cast<char *>(create_path(tmp_pool, apr_pstrcat(tmp_pool, name, TMP_SUFFIX, NULL)));
+    tmp_path = const_cast<char *>(create_path(tmp_pool, file_dir_,
+                                              apr_pstrcat(tmp_pool, name, TMP_SUFFIX, NULL)));
 
     if (apr_file_mktemp(tmp_file, tmp_path,
-                        APR_READ|APR_WRITE|APR_CREATE|APR_EXCL|APR_BINARY,
+                        APR_READ|APR_WRITE|APR_CREATE|APR_EXCL|APR_BINARY|
+                        (is_buffered ? APR_BUFFERED : 0),
                         tmp_pool) != APR_SUCCESS) {
         throw "åեǤޤǤ";
     }
@@ -507,24 +523,14 @@ const char *UploadItemList::create_tmp_path(apr_pool_t *tmp_pool, const char *na
     return tmp_path;
 }
 
-const char *UploadItemList::get_path_cache(apr_pool_t *tmp_pool)
+const char *UploadItemList::get_tmp_path_cache_path(apr_pool_t *tmp_pool, apr_file_t **cache_file)
 {
-    return create_path(tmp_pool, PATH_CACHE_FILENAME);
+    return create_tmp_path(tmp_pool, PATH_CACHE_FILENAME, true, cache_file);
 }
 
-const char *UploadItemList::get_header_cache(apr_pool_t *tmp_pool)
+const char *UploadItemList::get_tmp_header_cache_path(apr_pool_t *tmp_pool, apr_file_t **cache_file)
 {
-    return create_path(tmp_pool, HEADER_CACHE_FILENAME);
-}
-
-const char *UploadItemList::get_tmp_path_cache(apr_pool_t *tmp_pool, apr_file_t **cache_file)
-{
-    return create_tmp_path(tmp_pool, PATH_CACHE_FILENAME, cache_file);
-}
-
-const char *UploadItemList::get_tmp_header_cache(apr_pool_t *tmp_pool, apr_file_t **cache_file)
-{
-    return create_tmp_path(tmp_pool, HEADER_CACHE_FILENAME, cache_file);
+    return create_tmp_path(tmp_pool, HEADER_CACHE_FILENAME, false, cache_file);
 }
 
 void UploadItemList::read_cache(apr_pool_t *tmp_pool,
@@ -549,7 +555,6 @@ void UploadItemList::read_cache(apr_pool_t *tmp_pool,
 void UploadItemList::read_path_cache(apr_pool_t *tmp_pool, char **paths,
                                      apr_size_t *number)
 {
-    const char *cache_path;
     apr_file_t *cache_file;
     apr_mmap_t *file_map;
     apr_finfo_t info;
@@ -557,9 +562,7 @@ void UploadItemList::read_path_cache(apr_pool_t *tmp_pool, char **paths,
 
     *number = 0;
 
-    cache_path = get_path_cache(tmp_pool);
-
-    if (apr_file_open(&cache_file, cache_path,
+    if (apr_file_open(&cache_file, path_cache_path_,
                       APR_READ|APR_BINARY,
                       APR_OS_DEFAULT, tmp_pool) != APR_SUCCESS) {
         throw "ƥΥåե򳫤ޤǤ";
@@ -601,16 +604,13 @@ void UploadItemList::read_header_cache(apr_pool_t *tmp_pool,
                                        UploadItem::header **headers,
                                        apr_size_t *number)
 {
-    const char *cache_path;
     apr_file_t *cache_file;
     apr_mmap_t *file_map;
     apr_finfo_t info;
 
     *number = 0;
 
-    cache_path = get_header_cache(tmp_pool);
-
-    if (apr_file_open(&cache_file, cache_path,
+    if (apr_file_open(&cache_file, header_cache_path_,
                       APR_READ|APR_BINARY,
                       APR_OS_DEFAULT, tmp_pool) != APR_SUCCESS) {
         throw "ƥΥåե򳫤ޤǤ";
@@ -659,13 +659,11 @@ void UploadItemList::write_cache()
 
 void UploadItemList::write_path_cache(apr_pool_t *tmp_pool)
 {
-    const char *cache_path;
     const char *tmp_cache_path;
     apr_file_t *cache_file;
     item_iterator i;
 
-    cache_path = get_path_cache(tmp_pool);
-    tmp_cache_path = get_tmp_path_cache(tmp_pool, &cache_file);
+    tmp_cache_path = get_tmp_path_cache_path(tmp_pool, &cache_file);
 
     for (i = begin(); i != end(); i++) {
         apr_file_printf(cache_file, "%s%c",
@@ -674,14 +672,13 @@ void UploadItemList::write_path_cache(apr_pool_t *tmp_pool)
 
     apr_file_close(cache_file);
 
-    if (apr_file_rename(tmp_cache_path, cache_path, tmp_pool) != APR_SUCCESS) {
+    if (apr_file_rename(tmp_cache_path, path_cache_path_, tmp_pool) != APR_SUCCESS) {
         throw "åե rename ˼Ԥޤ";
     }
 }
 
 void UploadItemList::write_header_cache(apr_pool_t *tmp_pool)
 {
-    const char *cache_path;
     const char *tmp_cache_path;
     apr_file_t *cache_file;
     apr_off_t offset;
@@ -689,8 +686,7 @@ void UploadItemList::write_header_cache(apr_pool_t *tmp_pool)
     UploadItem::header *header;
     item_iterator i;
 
-    cache_path = get_header_cache(tmp_pool);
-    tmp_cache_path = get_tmp_header_cache(tmp_pool, &cache_file);
+    tmp_cache_path = get_tmp_header_cache_path(tmp_pool, &cache_file);
 
     if (!empty()) {
         offset = sizeof(UploadItem::header)*size() - 1;
@@ -720,7 +716,7 @@ void UploadItemList::write_header_cache(apr_pool_t *tmp_pool)
 
     apr_file_close(cache_file);
 
-    if (apr_file_rename(tmp_cache_path, cache_path, tmp_pool) != APR_SUCCESS) {
+    if (apr_file_rename(tmp_cache_path, header_cache_path_, tmp_pool) != APR_SUCCESS) {
         throw "åե rename ˼Ԥޤ";
     }
 }
