@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemWriter.cpp 551 2005-09-01 07:38:16Z svn $
+ * $Id: UploadItemWriter.cpp 676 2005-09-14 14:43:48Z svn $
  *****************************************************************************/
 
 #include "UploadItemWriter.h"
@@ -73,13 +73,13 @@ void UploadItemWriter::write_date(UploadItem::header *header, const char *file_p
         throw "åץɥե̾ѹǤޤǤ";
     }
 
-    if (apr_file_open(&file, file_path, APR_WRITE|APR_BINARY,
+    if (apr_file_open(&file, file_path, APR_WRITE|APR_BINARY|APR_BUFFERED,
                       APR_OS_DEFAULT, pool_) != APR_SUCCESS) {
         throw "åץɥե򳫤ޤǤ";
     }
 
-    size = sizeof(UploadItem::header);
-    if (apr_file_write(file, header, &size) != APR_SUCCESS) {
+    if ((apr_file_write_full(file, header, sizeof(UploadItem::header), &size) != APR_SUCCESS) ||
+        (size != sizeof(UploadItem::header))) {
         throw "åץɥǡν񤭹ߤ˼Ԥޤ";
     }
 
