@@ -24,13 +24,14 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploaderConfig.cpp 643 2005-09-07 23:36:20Z svn $
+ * $Id: UploaderConfig.cpp 694 2005-09-17 13:54:06Z svn $
  *****************************************************************************/
 
+#define __STDC_CONSTANT_MACROS
+
 #include "mod_uploader_data.h"
 
 #include "UploaderConfig.h"
-#include "Auxiliary.h"
 #include "Misc.h"
 
 #include "apr_strings.h"
@@ -43,9 +44,9 @@
 #define DELETE(ptr) if (ptr != NULL) { delete ptr; ptr = NULL;}
 #define DESTROY(pool) if (pool != NULL) { apr_pool_destroy(pool); pool = NULL;}
 
-const apr_uint64_t UploaderConfig::TOTAL_FILE_SIZE_LIMIT= 10*1024*1024;
+const apr_uint64_t UploaderConfig::TOTAL_FILE_SIZE_LIMIT= 1024*1024*1024;
 const apr_size_t UploaderConfig::TOTAL_FILE_NUMBER_LIMIT= 1000;
-const apr_size_t UploaderConfig::MAX_FORM_FILE_SIZE     = 1*1024*1024;
+const apr_size_t UploaderConfig::MAX_FORM_FILE_SIZE     = 100*1024*1024;
 const apr_size_t UploaderConfig::MAX_FORM_TEXT_SIZE     = 1024;
 const apr_size_t UploaderConfig::MAX_FORM_ITEM_NUMBER   = 5;
 const apr_size_t UploaderConfig::PER_PAGE_ITEM_NUMBER   = 50;
@@ -107,12 +108,13 @@ UploaderConfig::~UploaderConfig()
 void UploaderConfig::init(request_rec *r)
 {
     if (LIKELY(is_init_)) {
-        return update_list();
+        return update_list(r->pool);
     }
 
     pool_ = r->server->process->pool;
+
     // ϼΥɤ mod_uploader.cpp ֤ɡѥեޥ
-    // ͤƤ夭ޤ
+    // ͤƤ夤
     glock_ = (static_cast<sconfig *>(ap_get_module_config(r->server->module_config,
                                                           &uploader_module)))->upload_lock;
 
@@ -329,9 +331,9 @@ void UploaderConfig::load_list()
     remove_item_number = 0;
 }
 
-void UploaderConfig::update_list()
+void UploaderConfig::update_list(apr_pool_t *pool)
 {
-    if (LIKELY(!item_list->is_need_update() &&
+    if (LIKELY(!item_list->is_need_update(pool) &&
                (remove_item_number != ITEM_RELOAD_THRESHOLD))) {
         return;
     }
@@ -379,6 +381,10 @@ void UploaderConfig::global_lock()
 
 void UploaderConfig::global_unlock()
 {
+    // åץɥեιǥ쥯ȥ mtime ˴ŤȽ
+    // Ƥ뤱ɡUNIX Ǥ mtime ñ̤٤ʤΤɬס:-)
+    apr_sleep(APR_USEC_PER_SEC);
+
     if (UNLIKELY(apr_global_mutex_unlock(glock_) != APR_SUCCESS)) {
         throw "global_unlock() failed.";
     }
@@ -436,16 +442,14 @@ void UploaderConfig::load_template(apr_pool_t *pool, apr_pool_t **tmpl_pool,
 
 apr_time_t UploaderConfig::get_mtime(apr_pool_t *pool, const char *path)
 {
-    apr_time_t time;
+    apr_finfo_t info;
 
-    time = mtime(pool, path);
-
-    if (UNLIKELY(time == 0)) {
+    if (UNLIKELY(apr_stat(&info, path, APR_FINFO_MTIME, pool) != APR_SUCCESS)) {
         throw apr_psprintf(pool, "Can not get information of file/directory. (%s)",
                            path);
     }
 
-    return time;
+    return info.mtime;
 }
 
 
