@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: mod_uploader.cpp 662 2005-09-08 17:20:32Z svn $
+ * $Id: mod_uploader.cpp 686 2005-09-15 10:40:49Z svn $
  *****************************************************************************/
 
 #define __STDC_CONSTANT_MACROS
@@ -128,7 +128,7 @@ static apr_status_t attach_shm(apr_pool_t *pool, apr_shm_t **shm_data, void **da
     fprintf(stderr, "[" PACKAGE_NAME "] " format, str); fputc('\n', stderr)
 #else
 #define SERROR(format, str) \
-    ap_log_error(APLOG_MARK, APLOG_CRIT, 0, s, "[" PACKAGE_NAME "] " format, str)
+    ap_log_error(APLOG_MARK, APLOG_CRIT, 0, server, "[" PACKAGE_NAME "] " format, str)
 #define RERROR(format, str) \
     ap_log_rerror(APLOG_MARK, APLOG_CRIT, 0, r, "[" PACKAGE_NAME "] " format, str)
 #endif
@@ -317,7 +317,8 @@ int upload(request_rec *r, uconfig *config, const char *arg)
     address = get_client_address(r);
     if (is_poster_list_contain(r, address)) {
         return error(r, config,
-                     apr_psprintf(r->pool, "%u ô֤ϺƤǤޤ", POST_INTERVAL_SEC));
+                     apr_psprintf(r->pool, "%" APR_SIZE_T_FMT  " ô֤ϺƤǤޤ",
+                                  POST_INTERVAL_SEC));
     }
 
     upload_id = get_upload_id(r);
@@ -399,26 +400,22 @@ int progress_data(request_rec *r, uconfig *config, const char *arg)
 
     upload_id = get_upload_id(r);
 
-    ap_update_mtime(r, apr_time_now());
-
     // upload_id  0 λϥ顼
-    if (upload_id == 0) {
+    if (UNLIKELY(upload_id == 0)) {
         ap_rputs("e", r);
         return OK;
     }
 
     progress = get_progress(r, upload_id);
 
-    if (progress != get_progress_list(r)) {
-        ap_rprintf(r,
-                   "s %" APR_SIZE_T_FMT " %" APR_SIZE_T_FMT,
-                   progress->total_size,
-                   progress->read_size);
-
+    if (UNLIKELY(progress == get_progress_list(r))) {
+        ap_rputs("u", r);
         return OK;
     }
 
-    ap_rputs("u", r);
+    ap_rprintf(r,
+               "s %" APR_SIZE_T_FMT " %" APR_SIZE_T_FMT,
+               progress->total_size, progress->read_size);
     return OK;
 }
 
@@ -608,7 +605,7 @@ int redirect_top(request_rec *r)
 uconfig *get_config(request_rec *r)
 {
     return static_cast<uconfig *>(ap_get_module_config(r->per_dir_config,
-                                                            &uploader_module));
+                                                       &uploader_module));
 }
 
 apr_table_t *get_command_table(request_rec *r)
@@ -846,7 +843,7 @@ pprogress *get_progress(request_rec *r, apr_size_t upload_id)
     progress_list = get_progress_list(r);
 
     if (upload_id == 0) {
-        return &progress_list[0];
+        return progress_list;
     }
 
     for (i = 1; i < PROGRESS_LIST_NUMBER; i++) {
@@ -855,7 +852,7 @@ pprogress *get_progress(request_rec *r, apr_size_t upload_id)
         }
     }
 
-    return &progress_list[0];
+    return progress_list;
 }
 
 pprogress *add_progress_list(request_rec *r, apr_size_t upload_id)
@@ -868,12 +865,12 @@ pprogress *add_progress_list(request_rec *r, apr_size_t upload_id)
     progress_list = get_progress_list(r);
 
     if (upload_id == 0) {
-        return &progress_list[0];
+        return progress_list;
     }
 
     lock = get_progress_lock(r);
     if (apr_global_mutex_lock(lock) != APR_SUCCESS) {
-        return &progress_list[0];
+        return progress_list;
     }
 
     now = apr_time_now();
@@ -883,7 +880,7 @@ pprogress *add_progress_list(request_rec *r, apr_size_t upload_id)
     // progress_list[i].end_time == 1        : (åץ)
     // progress_list[i].end_time == ʳ : (åץɴλ)
 
-    // Ʊ upload_id 뤫åĤĸŤȥ
+    // Ʊ upload_id 뤫åĤĸŤȥ̤Ѥˡ
     for (i = 1; i < PROGRESS_LIST_NUMBER; i++) {
         if (progress_list[i].end_time == 0) {
             continue;
@@ -896,10 +893,11 @@ pprogress *add_progress_list(request_rec *r, apr_size_t upload_id)
         } else if (progress_list[i].upload_id == upload_id) {
             apr_global_mutex_unlock(lock);
 
-            return &progress_list[0];
+            return progress_list;
         }
     }
 
+    // ̤ѥȥõϿ
     for (i = 1; i < PROGRESS_LIST_NUMBER; i++) {
         if (progress_list[i].end_time == 0) {
             progress_list[i].upload_id  = upload_id;
@@ -915,7 +913,7 @@ pprogress *add_progress_list(request_rec *r, apr_size_t upload_id)
 
     apr_global_mutex_unlock(lock);
 
-    return &progress_list[0];
+    return progress_list;
 }
 
 pprogress *get_progress_list(request_rec *r)
@@ -998,7 +996,7 @@ poster *get_poster_list(request_rec *r)
 }
 
 apr_status_t create_lock(apr_pool_t *pool, apr_global_mutex_t **lock,
-                         const char *file_path, server_rec *s)
+                         const char *file_path, server_rec *server)
 {
     apr_status_t status;
     char error[MAX_ERROR_MESSAGE];
@@ -1025,7 +1023,7 @@ apr_status_t create_lock(apr_pool_t *pool, apr_global_mutex_t **lock,
 }
 
 apr_status_t create_shm(apr_pool_t *pool, apr_shm_t **shm_data, void **data,
-                        const char *file_path, apr_size_t size, server_rec *s)
+                        const char *file_path, apr_size_t size, server_rec *server)
 {
     apr_status_t status;
     char error[MAX_ERROR_MESSAGE];
@@ -1049,7 +1047,7 @@ apr_status_t create_shm(apr_pool_t *pool, apr_shm_t **shm_data, void **data,
 }
 
 apr_status_t attach_shm(apr_pool_t *pool, apr_shm_t **shm_data, void **data,
-                        const char *file_path, server_rec *s)
+                        const char *file_path, server_rec *server)
 {
     apr_status_t status;
     char error[MAX_ERROR_MESSAGE];
@@ -1078,6 +1076,11 @@ static int uploader_handler(request_rec *r)
     if (strcmp(r->handler, "uploader")) {
         return DECLINED;
     }
+
+    if (UNLIKELY(!ap_is_initial_req(r))) {
+        return DECLINED;
+    }
+
     config = get_config(r);
 
     try {
@@ -1097,6 +1100,14 @@ static apr_status_t finalize_server_config(void *data)
 
     config = static_cast<sconfig *>(data);
 
+    if (config->upload_lock != NULL) {
+        apr_global_mutex_destroy(config->upload_lock);
+        config->upload_lock = NULL;
+    }
+    if (config->progress_lock != NULL) {
+        apr_global_mutex_destroy(config->progress_lock);
+        config->progress_lock = NULL;
+    }
     if (config->poster_list_shm != NULL) {
         apr_shm_destroy(config->poster_list_shm);
         config->poster_list_shm = NULL;
@@ -1124,17 +1135,21 @@ static void *create_server_config(apr_pool_t *p, server_rec *s)
     sconfig *config;
     const char *tmp_dir;
 
-    config = static_cast<sconfig *>(apr_palloc(p, sizeof(sconfig)));
+    APR_PCALLOC(config, sconfig *, p, sizeof(sconfig));
 
     apr_temp_dir_get(&tmp_dir, p);
 
-    apr_filepath_merge(&(config->upload_lock_path), tmp_dir, UPLOAD_LOCK_NAME,
+    apr_filepath_merge(&(config->upload_lock_path), tmp_dir,
+                       apr_pstrcat(p, UPLOAD_LOCK_NAME, "_", s->server_hostname, NULL),
                        APR_FILEPATH_NOTABOVEROOT, p);
-    apr_filepath_merge(&(config->progress_lock_path), tmp_dir, PROGRESS_LOCK_NAME,
+    apr_filepath_merge(&(config->progress_lock_path), tmp_dir,
+                       apr_pstrcat(p, PROGRESS_LOCK_NAME, "_", s->server_hostname, NULL),
                        APR_FILEPATH_NOTABOVEROOT, p);
-    apr_filepath_merge(&(config->poster_list_path), tmp_dir, POSTER_LIST_NAME,
+    apr_filepath_merge(&(config->poster_list_path), tmp_dir,
+                       apr_pstrcat(p, POSTER_LIST_NAME, "_", s->server_hostname, NULL),
                        APR_FILEPATH_NOTABOVEROOT, p);
-    apr_filepath_merge(&(config->progress_list_path), tmp_dir, PROGRESS_LIST_NAME,
+    apr_filepath_merge(&(config->progress_list_path), tmp_dir,
+                       apr_pstrcat(p, PROGRESS_LIST_NAME, "_", s->server_hostname, NULL),
                        APR_FILEPATH_NOTABOVEROOT, p);
 
     config->poster_list_shm = NULL;
@@ -1147,7 +1162,7 @@ static void *create_dir_config(apr_pool_t *pool, char *dirspec)
 {
     uconfig *config;
 
-    APR_PALLOC(config, uconfig *, pool, sizeof(uconfig));
+    APR_PCALLOC(config, uconfig *, pool, sizeof(uconfig));
     new(config) uconfig;
 
     apr_pool_cleanup_register(pool, config, finalize_dir_config, apr_pool_cleanup_null);
@@ -1155,11 +1170,79 @@ static void *create_dir_config(apr_pool_t *pool, char *dirspec)
     return config;
 }
 
+static apr_status_t post_config_server(apr_pool_t *pool, server_rec *server)
+{
+    sconfig *config;
+
+    config = static_cast<sconfig *>(ap_get_module_config(server->module_config, &uploader_module));
+
+    if (create_lock(pool, &(config->upload_lock), config->upload_lock_path,
+                    server) != APR_SUCCESS) {
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
+    if (create_lock(pool, &(config->progress_lock), config->progress_lock_path,
+                    server) != APR_SUCCESS) {
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
+    if (create_shm(pool, &(config->poster_list_shm),
+                   reinterpret_cast<void **>(&config->poster_list),
+                   config->poster_list_path, sizeof(poster)*POSTER_LIST_NUMBER,
+                   server) != APR_SUCCESS) {
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
+    if (create_shm(pool, &(config->progress_list_shm),
+                   reinterpret_cast<void **>(&(config->progress_list)),
+                   config->progress_list_path, sizeof(pprogress)*PROGRESS_LIST_NUMBER,
+                   server) != APR_SUCCESS) {
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
+    apr_pool_cleanup_register(pool, config, finalize_server_config, apr_pool_cleanup_null);
+
+    return OK;
+}
+
+static void child_init_server(apr_pool_t *pool, server_rec *server)
+{
+    sconfig *config;
+
+    config = static_cast<sconfig *>(ap_get_module_config(server->module_config, &uploader_module));
+
+    if (apr_global_mutex_child_init(&(config->upload_lock),
+                                    config->upload_lock_path, pool) != APR_SUCCESS) {
+        SERROR("Can not attach to global mutex (%s).", config->upload_lock_path);
+
+        return;
+    }
+
+    if (apr_global_mutex_child_init(&(config->progress_lock),
+                                    config->progress_lock_path, pool) != APR_SUCCESS) {
+        SERROR("Can not attach to global mutex (%s).", config->progress_lock_path);
+
+        return;
+    }
+
+    if (attach_shm(pool, &(config->poster_list_shm),
+                   reinterpret_cast<void **>(&config->poster_list),
+                   config->poster_list_path, server) != APR_SUCCESS) {
+        return;
+    }
+
+    if (attach_shm(pool, &(config->progress_list_shm),
+                   reinterpret_cast<void **>(&config->progress_list),
+                   config->progress_list_path, server) != APR_SUCCESS) {
+        return;
+    }
+}
+
 static int post_config(apr_pool_t *pconf, apr_pool_t *plog,
                        apr_pool_t *ptemp, server_rec *s)
 {
-    sconfig *config;
     void *user_data;
+    apr_status_t status;
 
     apr_pool_userdata_get(&user_data, USER_DATA_KEY, s->process->pool);
     if (user_data == NULL) {
@@ -1168,66 +1251,27 @@ static int post_config(apr_pool_t *pconf, apr_pool_t *plog,
         return OK;
     }
 
-    config = static_cast<sconfig *>(ap_get_module_config(s->module_config, &uploader_module));
+    do {
+        status = post_config_server(plog, s);
 
-    if (create_lock(pconf, &(config->upload_lock), config->upload_lock_path,
-                    s) != APR_SUCCESS) {
-        return HTTP_INTERNAL_SERVER_ERROR;
-    }
-
-    if (create_lock(pconf, &(config->progress_lock), config->progress_lock_path,
-                    s) != APR_SUCCESS) {
-        return HTTP_INTERNAL_SERVER_ERROR;
-    }
-
-    if (create_shm(pconf, &(config->poster_list_shm),
-                   reinterpret_cast<void **>(&config->poster_list),
-                   config->poster_list_path, sizeof(poster)*POSTER_LIST_NUMBER,
-                   s) != APR_SUCCESS) {
-        return HTTP_INTERNAL_SERVER_ERROR;
-    }
-
-    if (create_shm(pconf, &(config->progress_list_shm),
-                   reinterpret_cast<void **>(&(config->progress_list)),
-                   config->progress_list_path, sizeof(pprogress)*PROGRESS_LIST_NUMBER,
-                   s) != APR_SUCCESS) {
-        return HTTP_INTERNAL_SERVER_ERROR;
-    }
+        if (status != OK) {
+            return status;
+        }
+    } while ((s = s->next) != NULL);
 
     // windows Ǥ򤹤ȥӥ̾ѤäƤޤ餷
 #ifndef WIN32
     ap_add_version_component(pconf, PACKAGE_NAME "/" PACKAGE_VERSION);
 #endif
 
-    apr_pool_cleanup_register(pconf, config, finalize_server_config, apr_pool_cleanup_null);
-
     return OK;
 }
 
 static void child_init(apr_pool_t *p, server_rec *s)
 {
-    sconfig *config;
-
-    config = static_cast<sconfig *>(ap_get_module_config(s->module_config, &uploader_module));
-
-    if (apr_global_mutex_child_init(&(config->upload_lock),
-                                    config->upload_lock_path, p) != APR_SUCCESS) {
-        SERROR("Can not attach to global mutex (%s).", config->upload_lock_path);
-
-        return;
-    }
-
-    if (attach_shm(p, &(config->poster_list_shm),
-                   reinterpret_cast<void **>(&config->poster_list),
-                   config->poster_list_path, s) != APR_SUCCESS) {
-        return;
-    }
-
-    if (attach_shm(p, &(config->progress_list_shm),
-                   reinterpret_cast<void **>(&config->progress_list),
-                   config->progress_list_path, s) != APR_SUCCESS) {
-        return;
-    }
+    do {
+        child_init_server(p, s);
+    } while ((s = s->next) != NULL);
 }
 
 static const char *set_url(cmd_parms *parms, void *mconfig, char *arg)
